@@ -6,7 +6,7 @@ use Devel::Confess 'color';
 use Cwd 'getcwd';
 use warnings FATAL => 'all';
 package SimpleFlow;
-our $VERSION = 0.02;
+our $VERSION = 0.03;
 use Term::ANSIColor;
 use Scalar::Util 'openhandle';
 use DDP {output => 'STDOUT', array_max => 10, show_memsize => 0};
@@ -15,11 +15,21 @@ use Cwd 'getcwd';
 use warnings FATAL => 'all';
 use Capture::Tiny 'capture';
 use Exporter 'import';
-our @EXPORT = qw(task);
+our @EXPORT = qw(say2 task);
+
+sub say2 { # say to both command line and
+	my ($msg, $fh) = @_;
+	my $current_sub = (split(/::/,(caller(0))[3]))[-1]; # https://stackoverflow.com/questions/2559792/how-can-i-get-the-name-of-the-current-subroutine-in-perl
+	if (not openhandle($fh)) {
+		die "the filehandle given to $current_sub with \"$msg\" isn't actually a filehandle";
+	}
+	say $msg;
+	say $fh $msg;
+}
 
 sub task {
 	my ($args) = @_;
-	my $current_sub = (split(/::/,(caller(0))[3]))[-1]; # https://stackoverflow.com/questions/2559792/how-can-i-get-the-name-of-the-current-subroutine-in-perl
+	my $current_sub = (split(/::/,(caller(0))[3]))[-1];
 	unless (ref $args eq 'HASH') {
 		die "args must be given as a hash ref, e.g. \"$current_sub({ data => \@blah })\"";
 	}
@@ -35,6 +45,7 @@ sub task {
 		'die',			  # die if not successful; 0 or 1
 		'input.files',   # check for input files; SCALAR or ARRAY
 		'log.fh',        # print to filehandle
+		'note',          # a note for the log
 		'overwrite',     # 
 		'output.files'	  # product files that need to be checked; can be scalar or array
 	);
@@ -104,6 +115,9 @@ sub task {
 		overwrite       => $args->{overwrite},
 		'output.files' => [@output_files],
 	);
+	if (defined $args->{note}) {
+		$r{note} = $args->{note};
+	}
 	if (defined $args->{'input.files'}) {
 		$r{'input.files'} = $args->{'input.files'};
 		$r{'input.file.size'} = \%input_file_size;
@@ -113,7 +127,7 @@ sub task {
 		say colored(['black on_green'], "\"$args->{cmd}\"\n") . ' has been done before';
 		$r{done} = 'before';
 		$r{'output.file.size'} = \%output_file_size;
-		p(%r, output => $args->{'log.fh'}, show_memsize => 0) if defined $args->{'log.fh'};
+		p(%r, output => $args->{'log.fh'}) if defined $args->{'log.fh'};
 		return \%r;
 	}
 	($r{stdout}, $r{stderr}, $r{'exit'}) = capture {
@@ -126,9 +140,12 @@ sub task {
 	my @missing_output_files = grep {not -f -r $_} @output_files;
 	if (scalar @missing_output_files > 0) {
 		say STDERR "this input to $current_sub:";
-		p $args;
+		say {$args->{'log.fh'}} "this input to $current_sub:" if defined $args->{'log.fh'};
+		p($args, output => $args->{'log.fh'}) if defined $args->{'log.fh'};
 		say STDERR 'has these files missing:';
+		say {$args->{'log.fh'}} 'has these files missing:' if defined $args->{'log.fh'};
 		p @missing_output_files;
+		p(@missing_output_files, output => $args->{'log.fh'}) if defined $args->{'log.fh'};
 		die 'those above files should be made but are missing';
 	}
 	%output_file_size = map {$_ => -s $_} @output_files;
@@ -138,7 +155,7 @@ sub task {
 		p @files_with_zero_size;
 		warn 'the above output files have 0 size.';
 	}
-	p(%r, output => $args->{'log.fh'}, show_memsize => 0) if defined $args->{'log.fh'};
+	p(%r, output => $args->{'log.fh'}) if defined $args->{'log.fh'};
 	if (($args->{'die'} eq 'true') && ($r{'exit'} != 0)) {
 		p %r;
 		die "$args->{cmd} failed"
@@ -167,9 +184,10 @@ All tasks return a hash, showing at a minimum 1) exit code, 2) the directory tha
 
 the only required key/argument is `cmd`, but other arguments are possible:
 
-    die			  # die if not successful; 0 or 1
+    die          # die if not successful; 'true' or 'false'
     input.files  # check for input files before running; SCALAR or ARRAY
     log.fh       # print to filehandle
+    note         # a note for the log
     overwrite    # overwrite previously existing files: "true" or "false"
     output.files # product files that need to be checked; SCALAR or ARRAY
 
@@ -177,9 +195,10 @@ You may wish to output results to a logfile using a previously opened filehandle
 
     my ($fh, $fname) = tempfile( UNLINK => 0, DIR => '/tmp');
     my $t = task({
-    	cmd            => 'which ln',
-    	'log.fh'       => $fh,
-    	'output.files' => $fname,
-    	overwrite      => 1
+        cmd            => 'which ln',
+        'log.fh'       => $fh,
+        'note'         => 'testing where ln comes from',
+        'output.files' => $fname,
+        overwrite      => 1
     });
     close $fh;
